from lalgebra.point import Point
import math

class Line:
	coord = property(lambda self: f'(({self.a.x}, {self.a.y}), ({self.b.x}, {self.b.y}))')
	midpoint = property(lambda self: Point((self.a.x + self.b.x) / 2, (self.a.y + self.b.y) / 2))
	gradient = property(lambda self: (self.a.y - self.b.y) / (self.a.x - self.b.x))
	pgrad = property(lambda self: -1 / self.gradient)
	edist = property(lambda self: math.dist([self.a.x, self.a.y], [self.b.x, self.b.y]))
	mdist = property(lambda self: abs(self.a.x-self.b.x) + abs(self.a.y-self.b.y))

	def __init__(self, a, b):
		self.a = a
		self.b = b

	def lineEqn(self):
		try:
			m = self.gradient
		except ZeroDivisionError:
			return f'x = {self.a.x}'
		c = self.a.y - (m*self.a.x)
		if m == 0:
			return f'y = {c}'
		else:
			if c == 0:
				return f'y = {m}x'
			elif c > 0:
				return f'y = {m}x + {c}'
			else:
				return f'y = {m}x - {c*-1}'

	def isParallel(self, l):
		return self.gradient == l.gradient

	def isPerp(self, l):
		return self.pgrad == l.gradient


from lalgebra.point import Point
from lalgebra.line import Line

class InvalidPolygonError(Exception):
    pass

class Polygon:
    vertices = []
    area = property(lambda self: sum([self.vertices[v] + self.vertices[v+1] for v in range(0, len(self.vertices)-1)]) + sum([-1*(self.vertices[v] + self.vertices[v-1]) for v in range(1, len(self.vertices))]))

    def __init__(self, vertices):
        if len(vertices) < 3: raise InvalidPolygonError("A polygon must have at least 3 vertices!")
        self.vertices = vertices


from distutils.core import setup

setup(
    name = 'lalgebra',
    packages = ['lalgebra'],
    version = '0.1.1',
    license='MIT',
    description = 'Includes basic support for points, lines and vectors',
    long_description="See github page.",
    long_description_content_type="text/markdown",
    author = 'Jonathan Tan Jiayi',
    author_email = 'jonathantanatlol@gmail.com',
    url = 'https://github.com/peaceknight05/linear',
    download_url = 'https://github.com/peaceknight05/linear/archive/v0.1.1.tar.gz',
    keywords = ['mathematics', 'linear algebra', 'algebra', 'topography', 'geometry', 'vectors', 'lines', 'points'],
    install_requires=[],
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable"
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
    ],
)


import math

class Vector:
	x = 0
	y = 0
	coord = property(lambda self: f'({self.x}, {self.y})')
	length = property(lambda self: math.sqrt(self.x**2 + self.y**2))

	def __init__(self, x, y):
		self.x = x
		self.y = y

	def term(self, v, w):
		bc = -(v.x * w.y) / v.y + w.x
		b = ((self.x) - (v.x * self.y) / v.y) / bc
		ac = v.x
		a = ((self.x) - (w.x * b)) / ac
		return (a, b)

	def smult(self, s):
		return Vector(self.x * s, self.y * s)

	def vadd(self, v):
		return Vector(self.x + v.x, self.y + v.y)

	def dprod(self, v):
		return self.x * v.x + self.y * v.y

	def angle(self, v):
		t = self.dprod(v)/(self.length()*v.length())
		t = math.acos(t)
		return math.degrees(t)

	def scom(self, v):
		return (self.dprod(v))/v.length()
